# Copyright 2015 Alex Brandt
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

AUTHOR = 'Alex Brandt'
AUTHOR_EMAIL = 'alunduil@alunduil.com'
COPYRIGHT = '2015'
DESCRIPTION = 'A Study in Fixture Based Testing Frameworking'
LICENSE = 'Apache-2.0'
NAME = 'torment'
URL = 'https://github.com/kumoru/torment'
VERSION = '1.0.0'
